/*  v4l2ucp - A universal control panel for all V4L2 devices
    Copyright (C) 2005 Scott J. Bertin (scottbertin@yahoo.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */
#include <fcntl.h>
#include <sys/ioctl.h>
#include <cerrno>
#include <cstring>

#include <qscrollview.h>
#include <qfiledialog.h>
#include <qstring.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include <qmenubar.h>
#include <qmessagebox.h>

#include "v4l2controls.h"
#include "mainWindow.h"

MainWindow::MainWindow(QWidget *parent, const char *name, WFlags f) :
    QMainWindow(parent, name, f),
    timer(new QTimer(this)),
    fd(-1)
{
    QPopupMenu *menu = new QPopupMenu(this);
    menu->setCheckable(false);
    menu->insertItem("&Open", this, SLOT(fileOpen()), CTRL+Key_O);
    menu->insertItem("&Close", this, SLOT(close()), CTRL+Key_W);
    menu->insertSeparator();
    menu->insertItem("E&xit", qApp, SLOT(quit()), CTRL+Key_Q);
    
    menuBar()->insertItem("&File", menu);
    
    menu = new QPopupMenu(this);
    menu->setCheckable(true);
    menu->insertItem("Now", this, SIGNAL(updateNow()));
    menu->insertSeparator();
    id1s = menu->insertItem("1s", this, SLOT(timer1s()));
    id5s = menu->insertItem("5s", this, SLOT(timer5s()));
    id30s = menu->insertItem("30s", this, SLOT(timer30s()));
    id1m = menu->insertItem("1m", this, SLOT(timer1m()));
    id5m = menu->insertItem("5m", this, SLOT(timer5m()));
    idNever = menu->insertItem("Manual Only", this, SLOT(timerNever()));
    menu->setItemChecked(idNever, TRUE);
    updateMenu = menu;
    
    menuBar()->insertItem("&Updates", menu);
    
    menu = new QPopupMenu(this);
    menu->setCheckable(false);
    resetAllId = menu->insertItem("&All");
    resetMenu = menu;
    
    menuBar()->insertItem("&Reset", menu);

    menu = new QPopupMenu(this);
    menu->setCheckable(false);
    id1s = menu->insertItem("&About", this, SLOT(about()));
    id1s = menu->insertItem("About &Qt", this, SLOT(aboutQt()));
    
    menuBar()->insertItem("&Help", menu);
    
    QObject::connect( timer, SIGNAL(timeout()),
                      this, SIGNAL(updateNow()) );

}

void MainWindow::fileOpen()
{
    QString newfilename = QFileDialog::getOpenFileName( "/dev",
        "V4L2 Devices (video* vout* vbi* radio*);;"
        "Video Capture (video*);;"
        "Video Output (vout*);;"
        "VBI (vbi*);;"
        "Radio (radio*);;"
        "All (*)",
        this );
    if ( !newfilename.isEmpty() ) {
        MainWindow *w = openFile(newfilename);
        if(w)
            w->show();
    }
}

MainWindow *MainWindow::openFile(const char *fileName)
{
    int fd = open(fileName, O_RDWR, 0);
    if(fd < 0) {
        QString msg;
	msg.sprintf("Unable to open file %s\n%s", fileName, strerror(errno));
	QMessageBox::warning(NULL, "v4l2ucp: Unable to open file", msg, "OK");
        return NULL;
    }
    
    struct v4l2_capability cap;
    if(ioctl(fd, VIDIOC_QUERYCAP, &cap) == -1) {
        QString msg;
	msg.sprintf("%s is not a V4L2 device", fileName);
	QMessageBox::warning(NULL, "v4l2ucp: Not a V4L2 device", msg, "OK");
        return NULL;
    }
    
    MainWindow *mw = new MainWindow();
    mw->fd = fd;
    QString str("v4l2ucp - ");
    str.append(fileName);
    mw->setCaption(str);
    
    QScrollView *sv = new QScrollView(mw);
    QGrid *grid = new QGrid(4,sv->viewport());
    sv->addChild(grid);
    
    new QLabel("driver", grid);
    new QLabel((const char *)cap.driver, grid);
    new QLabel(grid);
    new QLabel(grid);

    new QLabel("card", grid);
    new QLabel((const char *)cap.card, grid);
    new QLabel(grid);
    new QLabel(grid);

    new QLabel("bus_info", grid);
    new QLabel((const char *)cap.bus_info, grid);
    new QLabel(grid);
    new QLabel(grid);
    

    str.sprintf("%d.%d.%d", cap.version>>16, (cap.version>>8)&0xff,
                cap.version&0xff);
    new QLabel("version", grid);
    new QLabel(str, grid);
    new QLabel(grid);
    new QLabel(grid);
    
    str.sprintf("0x%08x", cap.capabilities);
    new QLabel("capabilities", grid);
    new QLabel(str, grid);
    new QLabel(grid);
    new QLabel(grid);

    /* Check all the standard controls */
    for(int i=V4L2_CID_BASE; i<V4L2_CID_LASTP1; i++) {
        struct v4l2_queryctrl ctrl;
        ctrl.id = i;
        if(ioctl(fd, VIDIOC_QUERYCTRL, &ctrl) == 0) {
            mw->add_control(ctrl, fd, grid);
        }
    }
    
    /* Check any custom controls */
    for(int i=V4L2_CID_PRIVATE_BASE; ; i++) {
        struct v4l2_queryctrl ctrl;
        ctrl.id = i;
        if(ioctl(fd, VIDIOC_QUERYCTRL, &ctrl) == 0) {
            mw->add_control(ctrl, fd, grid);
        } else {
            break;
        }
    }
    
    mw->setCentralWidget(sv);
    return mw;
}

MainWindow::~MainWindow()
{
    if(fd >= 0)
        close(fd);
}

void MainWindow::add_control(struct v4l2_queryctrl &ctrl, int fd, QGrid *parent)
{
    QWidget *w = NULL;
    
    if(ctrl.flags & V4L2_CTRL_FLAG_DISABLED)
        return;
    
    new QLabel((const char *)ctrl.name, parent);
    
    switch(ctrl.type) {
        case V4L2_CTRL_TYPE_INTEGER:
            w = new V4L2IntegerControl(fd, ctrl, parent);
            break;
        case V4L2_CTRL_TYPE_BOOLEAN:
            w = new V4L2BooleanControl(fd, ctrl, parent);
            break;
        case V4L2_CTRL_TYPE_MENU:
            w = new V4L2MenuControl(fd, ctrl, parent);
            break;
        case V4L2_CTRL_TYPE_BUTTON:
            w = new V4L2ButtonControl(fd, ctrl, parent);
            break;
    }
    
    if(!w) {
        new QLabel("Unknown control", parent);
        new QLabel(parent);
        new QLabel(parent);
        return;
    }
    
    if(ctrl.flags & V4L2_CTRL_FLAG_GRABBED) {
        w->setEnabled(false);
    }

    QPushButton *pb;
    pb = new QPushButton("Update", parent);
    QObject::connect( pb, SIGNAL(clicked()), w, SLOT(updateStatus()) );
    QObject::connect( this, SIGNAL(updateNow()), w, SLOT(updateStatus()) );
    
    if(ctrl.type == V4L2_CTRL_TYPE_BUTTON) {
        new QLabel(parent);
    } else {
        pb = new QPushButton("Reset", parent);
        QObject::connect( pb, SIGNAL(clicked()), w, SLOT(resetToDefault()) );
        resetMenu->connectItem(resetAllId, w, SLOT(resetToDefault()) );
    }
}

void MainWindow::timer1s()
{
    updateMenu->setItemChecked(id1s, TRUE);
    updateMenu->setItemChecked(id5s, FALSE);
    updateMenu->setItemChecked(id30s, FALSE);
    updateMenu->setItemChecked(id1m, FALSE);
    updateMenu->setItemChecked(id5m, FALSE);
    updateMenu->setItemChecked(idNever, FALSE);
    timer->changeInterval(1000);
}

void MainWindow::timer5s()
{
    updateMenu->setItemChecked(id1s, FALSE);
    updateMenu->setItemChecked(id5s, TRUE);
    updateMenu->setItemChecked(id30s, FALSE);
    updateMenu->setItemChecked(id1m, FALSE);
    updateMenu->setItemChecked(id5m, FALSE);
    updateMenu->setItemChecked(idNever, FALSE);
    timer->changeInterval(5000);
}

void MainWindow::timer30s()
{
    updateMenu->setItemChecked(id1s, FALSE);
    updateMenu->setItemChecked(id5s, FALSE);
    updateMenu->setItemChecked(id30s, TRUE);
    updateMenu->setItemChecked(id1m, FALSE);
    updateMenu->setItemChecked(id5m, FALSE);
    updateMenu->setItemChecked(idNever, FALSE);
    timer->changeInterval(30000);
}

void MainWindow::timer1m()
{
    updateMenu->setItemChecked(id1s, FALSE);
    updateMenu->setItemChecked(id5s, FALSE);
    updateMenu->setItemChecked(id30s, FALSE);
    updateMenu->setItemChecked(id1m, TRUE);
    updateMenu->setItemChecked(id5m, FALSE);
    updateMenu->setItemChecked(idNever, FALSE);
    timer->changeInterval(60000);
}

void MainWindow::timer5m()
{
    updateMenu->setItemChecked(id1s, FALSE);
    updateMenu->setItemChecked(id5s, FALSE);
    updateMenu->setItemChecked(id30s, FALSE);
    updateMenu->setItemChecked(id1m, FALSE);
    updateMenu->setItemChecked(id5m, TRUE);
    updateMenu->setItemChecked(idNever, FALSE);
    timer->changeInterval(300000);
}

void MainWindow::timerNever()
{
    updateMenu->setItemChecked(id1s, FALSE);
    updateMenu->setItemChecked(id5s, FALSE);
    updateMenu->setItemChecked(id30s, FALSE);
    updateMenu->setItemChecked(id1m, FALSE);
    updateMenu->setItemChecked(id5m, FALSE);
    updateMenu->setItemChecked(idNever, TRUE);
    timer->stop();
}

void MainWindow::about()
{
    QMessageBox::about(this, "About", "v4l2ucp Version "PACKAGE_VERSION"\n\n"
        "This is a universal control panel for all V4L2 devices. The\n"
        "controls come directly from the driver. If they cause problems\n"
        "with your hardware, please contact the maintainer of the driver.\n\n"
        "Copyright (C) 2005 Scott J. Bertin\n\n"
        "This program is free software; you can redistribute it and/or modify\n"
        "it under the terms of the GNU General Public License as published by\n"
        "the Free Software Foundation; either version 2 of the License, or\n"
        "(at your option) any later version.\n\n"
        "This program is distributed in the hope that it will be useful,\n"
        "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
        "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
        "GNU General Public License for more details.\n\n"
        "You should have received a copy of the GNU General Public License\n"
        "along with this program; if not, write to the Free Software\n"
        "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n");
}

void MainWindow::aboutQt()
{
    QMessageBox::aboutQt(this);
}
